import logging
from typing import Optional, Type, List

from graphlit import Graphlit
from graphlit_api import exceptions, input_types, enums
from pydantic import BaseModel, Field

from ..base_tool import BaseTool
from ..exceptions import ToolException
from .. import helpers

logger = logging.getLogger(__name__)

class ContentRetrievalInput(BaseModel):
    search: str = Field(description="Text to search for within the knowledge base")
    types: Optional[List[enums.ContentTypes]] = Field(description="List of content types (i.e. FILE, PAGE, EMAIL, ISSUE, MESSAGE) to be returned from knowledge base, optional.", default=None)
    limit: Optional[int] = Field(description="Number of contents to return from search query, optional.", default=None)

class ContentRetrievalTool(BaseTool):
    name: str = "Graphlit content retrieval tool"
    description: str = """Accepts search text as string.
    Optionally accepts a list of content types (i.e. FILE, PAGE, EMAIL, ISSUE, MESSAGE) for filtering the result set.
    Retrieves contents based on similarity search from knowledge base.
    Returns extracted Markdown text and metadata from contents relevant to the search text.
    Can search through web pages, PDFs, audio transcripts, Slack messages, emails, or any unstructured data ingested into the knowledge base."""
    args_schema: Type[BaseModel] = ContentRetrievalInput

    graphlit: Graphlit = Field(None, exclude=True)
    search_type: Optional[enums.SearchTypes] = Field(None, exclude=True)

    model_config = {
        "arbitrary_types_allowed": True
    }

    def __init__(self, graphlit: Optional[Graphlit] = None, search_type: Optional[enums.SearchTypes] = None, **kwargs):
        """
        Initializes the ContentRetrievalTool.

        Args:
            graphlit (Optional[Graphlit]): An optional Graphlit instance to interact with the Graphlit API.
                If not provided, a new Graphlit instance will be created.
            search_type (Optional[SearchTypes]): An optional enum specifying the type of search to use: VECTOR, HYBRID or KEYWORD.
                If not provided, vector search will be used.
            **kwargs: Additional keyword arguments for the BaseTool superclass.
        """
        super().__init__(**kwargs)
        self.graphlit = graphlit or Graphlit()
        self.search_type = search_type

    async def _arun(self, search: str, types: Optional[List[enums.ContentTypes]] = None, limit: Optional[int] = None) -> Optional[str]:
        try:
            response = await self.graphlit.client.query_contents(
                filter=input_types.ContentFilter(
                    types=types,
                    search=search,
                    searchType=self.search_type if self.search_type is not None else enums.SearchTypes.HYBRID,
                    limit=limit if limit is not None else 10 # NOTE: default to 10 relevant contents
                )
            )

            if response.contents is None or response.contents.results is None:
                raise ToolException('Failed to retrieve contents.')

            logger.debug(f'ContentRetrievalTool: Retrieved [{len(response.contents.results)}] content(s) given search text [{search}].')

            results = []

            for content in response.contents.results:
                results.extend(helpers.format_content(content))

            text = "\n".join(results)

            return text
        except exceptions.GraphQLClientError as e:
            logger.error(str(e))
            raise ToolException(str(e)) from e

    def _run(self, search: str, types: Optional[List[enums.ContentTypes]] = None, limit: Optional[int] = None) -> Optional[str]:
        return helpers.run_async(self._arun, search, types, limit)



import logging
from typing import Optional, Type

from graphlit import Graphlit
from graphlit_api import exceptions, input_types, enums
from pydantic import BaseModel, Field

from ..base_tool import BaseTool
from ..exceptions import ToolException
from .. import helpers

logger = logging.getLogger(__name__)

class PersonRetrievalInput(BaseModel):
    search: str = Field(description="Text to search for within the knowledge base")
    email: Optional[str] = Field(description="Email of person to retrieve.", default=None)
    limit: Optional[int] = Field(description="Number of persons to return from search query, optional.", default=10)

class PersonRetrievalTool(BaseTool):
    name: str = "Graphlit person retrieval tool"
    description: str = """Accepts search text as string. Optionally, accepts person email as string.
    Retrieves persons based on similarity search from knowledge base.
    Returns metadata from persons relevant to the search text."""
    args_schema: Type[BaseModel] = PersonRetrievalInput

    graphlit: Graphlit = Field(None, exclude=True)
    search_type: Optional[enums.SearchTypes] = Field(None, exclude=True)

    model_config = {
        "arbitrary_types_allowed": True
    }

    def __init__(self, graphlit: Optional[Graphlit] = None, search_type: Optional[enums.SearchTypes] = None, **kwargs):
        """
        Initializes the PersonRetrievalTool.

        Args:
            graphlit (Optional[Graphlit]): An optional Graphlit instance to interact with the Graphlit API.
                If not provided, a new Graphlit instance will be created.
            search_type (Optional[SearchTypes]): An optional enum specifying the type of search to use: VECTOR, HYBRID or KEYWORD.
                If not provided, vector search will be used.
            **kwargs: Additional keyword arguments for the BaseTool superclass.
        """
        super().__init__(**kwargs)
        self.graphlit = graphlit or Graphlit()
        self.search_type = search_type

    async def _arun(self, search: str = None, email: Optional[str] = None, limit: Optional[int] = None) -> Optional[str]:
        try:
            response = await self.graphlit.client.query_persons(
                filter=input_types.PersonFilter(
                    search=search,
                    email=email,
                    searchType=self.search_type if self.search_type is not None else enums.SearchTypes.HYBRID,
                    limit=limit if limit is not None else 10 # NOTE: default to 10 relevant persons
                )
            )

            if response.persons is None or response.persons.results is None:
                return None

            logger.debug(f'PersonRetrievalTool: Retrieved [{len(response.persons.results)}] person(s) given search text [{search}].')

            results = []

            for person in response.persons.results:
                results.extend(helpers.format_person(person))

            text = "\n".join(results)

            return text
        except exceptions.GraphQLClientError as e:
            logger.error(str(e))
            raise ToolException(str(e)) from e

    def _run(self, search: str = None, email: Optional[str] = None, limit: Optional[int] = None) -> Optional[str]:
        return helpers.run_async(self._arun, search, email, limit)



from typing import Any, Dict, cast
from schema import Schema
from .base_tool import BaseTool

GriptapeBaseTool: Any = None

try:
    from griptape.tools import BaseTool as GriptapeBaseTool
    from griptape.utils.decorators import activity
    from griptape.artifacts import TextArtifact
except ImportError:
    GriptapeBaseTool = None

if GriptapeBaseTool:
    class GriptapeConverter(GriptapeBaseTool):
        """Tool to convert Graphlit tools into Griptape tools."""

        graphlit_tool: BaseTool

        @classmethod
        def from_tool(cls, tool: Any, **kwargs: Any) -> "GriptapeConverter":
            if not isinstance(tool, BaseTool):
                raise ValueError(f"Expected a Graphlit tool, got {type(tool)}")

            graphlit_tool = cast(BaseTool, tool)

            if graphlit_tool.args_schema is None:
                raise ValueError("Invalid arguments JSON schema.")

            # Create an instance of GriptapeConverter
            instance = cls(name=graphlit_tool.name, **kwargs)
            instance.graphlit_tool = graphlit_tool

            # Define the generate method dynamically
            def generate(self, params: Dict[str, Any]) -> TextArtifact:
                return TextArtifact(str(self.graphlit_tool.run(**params)))

            # Convert the tool's schema
            tool_schema = Schema(graphlit_tool.json_schema)

            # Decorate the generate method
            decorated_generate = activity(
                config={
                    "description": graphlit_tool.description,
                    "schema": tool_schema,
                }
            )(generate)

            # Attach the dynamically created method to the instance
            setattr(instance, "generate", decorated_generate)

            return instance
else:
    class GriptapeConverter:
        """Fallback GriptapeConverter if griptape is not installed."""

        @classmethod
        def from_tool(cls, tool: Any, **kwargs: Any) -> "GriptapeConverter":
            raise ImportError(
                "GriptapeConverter requires the griptape package. "
                "Install it using pip install graphlit-tools[griptape]."
            )



import logging
import time
import os
from typing import Optional, Type

from graphlit import Graphlit
from graphlit_api import exceptions, input_types, enums
from pydantic import BaseModel, Field

from ..base_tool import BaseTool
from ..exceptions import ToolException
from .. import helpers

logger = logging.getLogger(__name__)

class GoogleEmailIngestInput(BaseModel):
    search: Optional[str] = Field(description="Text to search for within ingested emails.", default=None)
    read_limit: Optional[int] = Field(description="Maximum number of emails from Google Email account to be read.", default=10)

class GoogleEmailIngestTool(BaseTool):
    name: str = "Graphlit Google Email ingest tool"
    description: str = """Ingests emails from Google Email account into knowledge base.
    Optionally accepts search text for searching within the ingested emails. If search text was not provided, all ingested emails will be returned.
    Returns extracted Markdown text and metadata from emails."""
    args_schema: Type[BaseModel] = GoogleEmailIngestInput

    graphlit: Graphlit = Field(None, exclude=True)

    workflow_id: Optional[str] = Field(None, exclude=True)
    correlation_id: Optional[str] = Field(None, exclude=True)

    model_config = {
        "arbitrary_types_allowed": True
    }

    def __init__(self, graphlit: Optional[Graphlit] = None, workflow_id: Optional[str] = None, correlation_id: Optional[str] = None, **kwargs):
        """
        Initializes the GmailIngestTool.

        Args:
            graphlit (Optional[Graphlit]): An optional Graphlit instance to interact with the Graphlit API.
                If not provided, a new Graphlit instance will be created.
            workflow_id (Optional[str]): ID for the workflow to use when ingesting emails. Defaults to None.
            correlation_id (Optional[str]): Correlation ID for tracking requests. Defaults to None.
            **kwargs: Additional keyword arguments for the BaseTool superclass.
        """
        super().__init__(**kwargs)
        self.graphlit = graphlit or Graphlit()
        self.workflow_id = workflow_id
        self.correlation_id = correlation_id

    async def _arun(self, search: Optional[str] = None, read_limit: Optional[int] = None) -> Optional[str]:
        feed_id = None

        refresh_token = os.environ['GOOGLE_EMAIL_REFRESH_TOKEN']

        if refresh_token is None:
            raise ToolException('Invalid Google Email refresh token. Need to assign GOOGLE_EMAIL_REFRESH_TOKEN environment variable.')

        client_id = os.environ['GOOGLE_EMAIL_CLIENT_ID']

        if client_id is None:
            raise ToolException('Invalid Google Email client identifier. Need to assign GOOGLE_EMAIL_CLIENT_ID environment variable.')

        client_secret = os.environ['GOOGLE_EMAIL_CLIENT_SECRET']

        if client_secret is None:
            raise ToolException('Invalid Google Email client secret. Need to assign GOOGLE_EMAIL_CLIENT_SECRET environment variable.')

        try:
            response = await self.graphlit.client.create_feed(
                feed=input_types.FeedInput(
                    name='Google Email',
                    type=enums.FeedTypes.EMAIL,
                    email=input_types.EmailFeedPropertiesInput(
                        type=enums.FeedServiceTypes.GOOGLE_EMAIL,
                        google=input_types.GoogleEmailFeedPropertiesInput(
                            type=enums.EmailListingTypes.PAST,
                            refreshToken=refresh_token,
                            clientId=client_id,
                            clientSecret=client_secret,
                        ),
                        readLimit=read_limit if read_limit is not None else 10
                    ),
                    workflow=input_types.EntityReferenceInput(id=self.workflow_id) if self.workflow_id is not None else None,
                ),
                correlation_id=self.correlation_id
            )

            feed_id = response.create_feed.id if response.create_feed is not None else None

            if feed_id is None:
                raise ToolException('Invalid feed identifier.')

            logger.debug(f'Created feed [{feed_id}].')

            # Wait for feed to complete, since ingestion happens asychronously
            done = False
            time.sleep(5)

            while not done:
                done = await helpers.is_feed_done(self.graphlit.client, feed_id)

                if done is None:
                    break

                if not done:
                    time.sleep(5)

            logger.debug(f'Completed feed [{feed_id}].')
        except exceptions.GraphQLClientError as e:
            logger.error(str(e))
            raise ToolException(str(e)) from e

        return await helpers.format_feed_contents(self.graphlit.client, feed_id, search)

    def _run(self, search: Optional[str] = None, read_limit: Optional[int] = None) -> str:
        return helpers.run_async(self._arun, search, read_limit)



import logging
from typing import Optional, Type

from graphlit import Graphlit
from graphlit_api import exceptions, input_types, enums
from pydantic import BaseModel, Field

from ..base_tool import BaseTool
from ..exceptions import ToolException
from .. import helpers

logger = logging.getLogger(__name__)

class GenerateSocialMediaPostsInput(BaseModel):
    text: str = Field(description="Text to be summarized into social media posts")
    count: Optional[int] = Field(description="Number of social media posts to be generated, optional", default=10)

class GenerateSocialMediaPostsTool(BaseTool):
    name: str = "Graphlit social media posts generation tool"
    description: str = """Accepts text as string.
    Optionally accepts the count of social media posts to be generated.
    Returns social media posts as text."""
    args_schema: Type[BaseModel] = GenerateSocialMediaPostsInput

    graphlit: Graphlit = Field(None, exclude=True)

    model_config = {
        "arbitrary_types_allowed": True
    }

    specification_id: Optional[str] = Field(None, exclude=True)
    correlation_id: Optional[str] = Field(None, exclude=True)

    def __init__(self, graphlit: Optional[Graphlit] = None, specification_id: Optional[str] = None,
                 correlation_id: Optional[str] = None, **kwargs):
        """
        Initializes the GenerateSocialMediaPostsTool.

        Args:
            graphlit (Optional[Graphlit]): Instance for interacting with the Graphlit API.
                Defaults to a new Graphlit instance if not provided.
            specification_id (Optional[str]): ID for the LLM specification. Defaults to None.
            correlation_id (Optional[str]): Correlation ID for tracking requests. Defaults to None.
            **kwargs: Additional keyword arguments for the BaseTool superclass.
        """
        super().__init__(**kwargs)
        self.graphlit = graphlit or Graphlit()
        self.specification_id = specification_id
        self.correlation_id = correlation_id

    async def _arun(self, text: str, count: Optional[int] = None) -> str:
        try:
            response = await self.graphlit.client.summarize_text(
                text=text,
                summarization=input_types.SummarizationStrategyInput(
                    type=enums.SummarizationTypes.POSTS,
                    items=count if count is not None else 10,
                    specification=input_types.EntityReferenceInput(id=self.specification_id) if self.specification_id is not None else None,
                ),
                correlation_id=self.correlation_id
            )

            if response.summarize_text is None or response.summarize_text.items is None:
                raise ToolException('Failed to generate social media posts.')

            items = response.summarize_text.items

            item = items[0] if response.summarize_text.items is not None and len(response.summarize_text.items) > 0 else None

            return item.text
        except exceptions.GraphQLClientError as e:
            logger.error(str(e))
            print(str(e))
            raise ToolException(str(e)) from e
        except Exception as e:
            logger.error(str(e))
            print(str(e))
            raise ToolException(str(e)) from e

    def _run(self, text: str, count: Optional[int] = None) -> str:
        return helpers.run_async(self._arun, text, count)



import logging
import time
import os
from typing import Optional, Type

from graphlit import Graphlit
from graphlit_api import exceptions, input_types, enums
from pydantic import BaseModel, Field

from ..base_tool import BaseTool
from ..exceptions import ToolException
from .. import helpers

logger = logging.getLogger(__name__)

class SlackIngestInput(BaseModel):
    channel_name: str = Field(description="Slack channel name")
    search: Optional[str] = Field(description="Text to search for within ingested messages.", default=None)
    read_limit: Optional[int] = Field(description="Maximum number of messages from Slack channel to be read.", default=10)

class SlackIngestTool(BaseTool):
    name: str = "Graphlit Slack ingest tool"
    description: str = """Ingests messages from Slack channel into knowledge base.
    Accepts Slack channel name.
    Optionally accepts search text for searching within the ingested messages. If search text was not provided, all ingested messages will be returned.
    Returns extracted Markdown text and metadata from messages."""
    args_schema: Type[BaseModel] = SlackIngestInput

    graphlit: Graphlit = Field(None, exclude=True)

    workflow_id: Optional[str] = Field(None, exclude=True)
    correlation_id: Optional[str] = Field(None, exclude=True)

    model_config = {
        "arbitrary_types_allowed": True
    }

    def __init__(self, graphlit: Optional[Graphlit] = None, workflow_id: Optional[str] = None, correlation_id: Optional[str] = None, **kwargs):
        """
        Initializes the SlackIngestTool.

        Args:
            graphlit (Optional[Graphlit]): An optional Graphlit instance to interact with the Graphlit API.
                If not provided, a new Graphlit instance will be created.
            workflow_id (Optional[str]): ID for the workflow to use when ingesting messages. Defaults to None.
            correlation_id (Optional[str]): Correlation ID for tracking requests. Defaults to None.
            **kwargs: Additional keyword arguments for the BaseTool superclass.
        """
        super().__init__(**kwargs)
        self.graphlit = graphlit or Graphlit()
        self.workflow_id = workflow_id
        self.correlation_id = correlation_id

    async def _arun(self, channel_name: str, search: Optional[str] = None, read_limit: Optional[int] = None) -> Optional[str]:
        feed_id = None

        token = os.environ['SLACK_BOT_TOKEN']

        if token is None:
            raise ToolException('Invalid Slack bot token. Need to assign SLACK_BOT_TOKEN environment variable.')

        try:
            response = await self.graphlit.client.create_feed(
                feed=input_types.FeedInput(
                    name='Slack',
                    type=enums.FeedTypes.SLACK,
                    slack=input_types.SlackFeedPropertiesInput(
                        type=enums.FeedListingTypes.PAST,
                        channel=channel_name,
                        token=token,
                        includeAttachments=True,
                        readLimit=read_limit if read_limit is not None else 10
                    ),
                    workflow=input_types.EntityReferenceInput(id=self.workflow_id) if self.workflow_id is not None else None,
                ),
                correlation_id=self.correlation_id
            )

            feed_id = response.create_feed.id if response.create_feed is not None else None

            if feed_id is None:
                raise ToolException('Invalid feed identifier.')

            logger.debug(f'Created feed [{feed_id}].')

            # Wait for feed to complete, since ingestion happens asychronously
            done = False
            time.sleep(5)

            while not done:
                done = await helpers.is_feed_done(self.graphlit.client, feed_id)

                if done is None:
                    break

                if not done:
                    time.sleep(5)

            logger.debug(f'Completed feed [{feed_id}].')
        except exceptions.GraphQLClientError as e:
            logger.error(str(e))
            raise ToolException(str(e)) from e

        return await helpers.format_feed_contents(self.graphlit.client, feed_id, search)

    def _run(self, channel_name: str, search: Optional[str] = None, read_limit: Optional[int] = None) -> str:
        return helpers.run_async(self._arun, channel_name, search, read_limit)



import logging
from typing import Optional, Type

from graphlit import Graphlit
from graphlit_api import exceptions
from pydantic import BaseModel, Field

from ..base_tool import BaseTool
from ..exceptions import ToolException
from .. import helpers

logger = logging.getLogger(__name__)

class WebMapInput(BaseModel):
    url: str = Field(description="URL of the web page to be mapped")

class WebMapTool(BaseTool):
    name: str = "Graphlit web map tool"
    description: str = """Accepts web page URL as string.
    Enumerates the web pages at or beneath the provided URL using web sitemap.
    Returns list of mapped URIs from web site."""
    args_schema: Type[BaseModel] = WebMapInput

    graphlit: Graphlit = Field(None, exclude=True)

    workflow_id: Optional[str] = Field(None, exclude=True)
    correlation_id: Optional[str] = Field(None, exclude=True)

    model_config = {
        "arbitrary_types_allowed": True
    }

    def __init__(self, graphlit: Optional[Graphlit] = None, correlation_id: Optional[str] = None, **kwargs):
        super().__init__(**kwargs)
        self.graphlit = graphlit or Graphlit()
        self.correlation_id = correlation_id

    async def _arun(self, url: str) -> Optional[str]:
        try:
            response = await self.graphlit.client.map_web(
                uri=url,
                correlation_id=self.correlation_id
            )

            results = response.map_web.results if response.map_web is not None else None

            if results is not None:
                logger.debug(f'Completed web map, found [{len(results)}] results.')

                return '\n'.join(result for result in results) if results is not None else None
            else:
                return None
        except exceptions.GraphQLClientError as e:
            logger.error(str(e))
            raise ToolException(str(e)) from e

    def _run(self, url: str) -> Optional[str]:
        return helpers.run_async(self._arun, url)



import os
from setuptools import setup, find_packages

# Read the content of your README file
with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

version = os.getenv('PACKAGE_VERSION', '1.0.0')

setup(
    name='graphlit-tools',
    version=version,
    packages=find_packages(),
    install_requires=[
        'graphlit-client'        
    ],
    extras_require={
        "crewai": ["crewai"],  # Extras for CrewAI support
        "griptape": ["griptape"]  # Extras for Griptape support
    },
    python_requires='>=3.10',
    author='Unstruk Data Inc.',
    author_email='questions@graphlit.com',
    description='Graphlit Agent Tools',
    url='https://github.com/graphlit/graphlit-tools-python/',
    long_description=long_description,
    long_description_content_type="text/markdown",
)



import logging
import os
import base64
import mimetypes
from typing import Optional, Type

from graphlit import Graphlit
from graphlit_api import exceptions
from pydantic import BaseModel, Field

from ..base_tool import BaseTool
from ..exceptions import ToolException
from .. import helpers

logger = logging.getLogger(__name__)

class LocalIngestInput(BaseModel):
    file_path: str = Field(description="Path of local file to be ingested into knowledge base")

class LocalIngestTool(BaseTool):
    name: str = "Graphlit local file ingest tool"
    description: str = """Ingests content from local file.
    Returns extracted Markdown text and metadata from content.
    Can ingest individual Word documents, PDFs, audio recordings, videos, images, or any other unstructured data."""
    args_schema: Type[BaseModel] = LocalIngestInput

    graphlit: Graphlit = Field(None, exclude=True)

    workflow_id: Optional[str] = Field(None, exclude=True)
    correlation_id: Optional[str] = Field(None, exclude=True)

    model_config = {
        "arbitrary_types_allowed": True
    }

    def __init__(self, graphlit: Optional[Graphlit] = None, workflow_id: Optional[str] = None, correlation_id: Optional[str] = None, **kwargs):
        """
        Initializes the LocalIngestTool.

        Args:
            graphlit (Optional[Graphlit]): An optional Graphlit instance to interact with the Graphlit API.
                If not provided, a new Graphlit instance will be created.
            workflow_id (Optional[str]): ID for the workflow to use when ingesting files. Defaults to None.
            correlation_id (Optional[str]): Correlation ID for tracking requests. Defaults to None.
            **kwargs: Additional keyword arguments for the BaseTool superclass.
        """
        super().__init__(**kwargs)
        self.graphlit = graphlit or Graphlit()
        self.workflow_id = workflow_id
        self.correlation_id = correlation_id

    async def _arun(self, file_path: str) -> Optional[str]:
        content_id = None

        try:
            file_name = os.path.basename(file_path)
            content_name, _ = os.path.splitext(file_name)

            mime_type = mimetypes.guess_type(file_name)[0]

            if mime_type is None:
                logger.error(f'Failed to infer MIME type from file [{file_name}].')
                raise ToolException(f'Failed to infer MIME type from file [{file_name}].')

            with open(file_path, "rb") as file:
                file_content = file.read()

            base64_content = base64.b64encode(file_content).decode('utf-8')

            response = await self.graphlit.client.ingest_encoded_file(content_name, base64_content, mime_type, is_synchronous=True)

            content_id = response.ingest_encoded_file.id if response.ingest_encoded_file is not None else None
        except exceptions.GraphQLClientError as e:
            logger.error(str(e))
            raise ToolException(str(e)) from e

        if content_id is None:
            raise ToolException('Invalid content identifier.')

        try:
            response = await self.graphlit.client.get_content(
                id=content_id
            )

            if response.content is None:
                raise ToolException(f'Failed to get content [{content_id}].')

            logger.debug(f'LocalIngestTool: Retrieved content by ID [{content_id}].')

            results = helpers.format_content(response.content)

            text = "\n".join(results)

            return text
        except exceptions.GraphQLClientError as e:
            logger.error(str(e))
            raise ToolException(str(e)) from e

    def _run(self, file_path: str) -> Optional[str]:
        return helpers.run_async(self._arun, file_path)



import logging
import json
from typing import Optional, Type

from graphlit import Graphlit
from graphlit_api import exceptions, input_types, enums
from pydantic import BaseModel, Field

from ..base_tool import BaseTool
from ..exceptions import ToolException
from .. import helpers

logger = logging.getLogger(__name__)

class ExtractURLInput(BaseModel):
    url: str = Field(description="URL of cloud-hosted file to be ingested into knowledge base")
    model_schema: str = Field(description="Pydantic model JSON schema which describes the data which will be extracted. JSON schema needs be of type 'object' and include 'properties' and 'required' fields.")
    prompt: Optional[str] = Field(description="Text prompt which is provided to LLM to guide data extraction, optional.", default=None)

class ExtractURLTool(BaseTool):
    name: str = "Graphlit JSON URL data extraction tool"
    description: str = """Extracts JSON data from ingested file using LLM.
    Accepts URL to be ingested, and JSON schema of Pydantic model to be extracted into. JSON schema needs be of type 'object' and include 'properties' and 'required' fields.
    Returns extracted JSON from file."""
    args_schema: Type[BaseModel] = ExtractURLInput

    graphlit: Graphlit = Field(None, exclude=True)

    workflow_id: Optional[str] = Field(None, exclude=True)
    specification_id: Optional[str] = Field(None, exclude=True)
    correlation_id: Optional[str] = Field(None, exclude=True)

    model_config = {
        "arbitrary_types_allowed": True
    }

    def __init__(self, graphlit: Optional[Graphlit] = None, workflow_id: Optional[str] = None, specification_id: Optional[str] = None, correlation_id: Optional[str] = None, **kwargs):
        """
        Initializes the ExtractURLTool.

        Args:
            graphlit (Optional[Graphlit]): An optional Graphlit instance to interact with the Graphlit API.
                If not provided, a new Graphlit instance will be created.
            workflow_id (Optional[str]): ID for the workflow to use when ingesting files. Defaults to None.
            specification_id (Optional[str]): ID for the LLM specification to use. Defaults to None.
            correlation_id (Optional[str]): Correlation ID for tracking requests. Defaults to None.
            **kwargs: Additional keyword arguments for the BaseTool superclass.
        """
        super().__init__(**kwargs)
        self.graphlit = graphlit or Graphlit()
        self.workflow_id = workflow_id
        self.specification_id = specification_id
        self.correlation_id = correlation_id

    async def _arun(self, url: str, model_schema: str, prompt: Optional[str] = None) -> Optional[str]:
        content_id = None

        try:
            response = await self.graphlit.client.ingest_uri(
                uri=url,
                workflow=input_types.EntityReferenceInput(id=self.workflow_id) if self.workflow_id is not None else None,
                is_synchronous=True,
                correlation_id=self.correlation_id
            )

            content_id = response.ingest_uri.id if response.ingest_uri is not None else None
        except exceptions.GraphQLClientError as e:
            logger.error(str(e))
            raise ToolException(str(e)) from e

        if content_id is None:
            raise ToolException('Invalid content identifier.')

        text = None

        try:
            response = await self.graphlit.client.get_content(
                id=content_id
            )

            if response.content is None:
                raise ToolException(f'Failed to get content [{content_id}].')

            logger.debug(f'ExtractURLTool: Retrieved content by ID [{content_id}].')

            results = helpers.format_content(response.content)

            text = "\n".join(results)
        except exceptions.GraphQLClientError as e:
            logger.error(str(e))
            raise ToolException(str(e)) from e

        if text is None:
            raise ToolException(f'Found no text to be extracted from content [{content_id}].')

        default_name = "extract_pydantic_model"

        default_prompt = """
        Extract data using the tools provided.
        """

        try:
            response = await self.graphlit.client.extract_text(
                specification=input_types.EntityReferenceInput(id=self.specification_id) if self.specification_id is not None else None,
                tools=[input_types.ToolDefinitionInput(name=default_name, schema=model_schema)],
                prompt=default_prompt if prompt is None else prompt,
                text=text,
                text_type=enums.TextTypes.MARKDOWN,
                correlation_id=self.correlation_id
            )

            if response.extract_text is None:
                raise ToolException('Failed to extract text.')

            extractions = response.extract_text

            json_array = json.loads('[' + ','.join(extraction.value for extraction in extractions) + ']')

            return json.dumps(json_array, indent=4)
        except exceptions.GraphQLClientError as e:
            logger.error(str(e))
            print(str(e))
            raise ToolException(str(e)) from e
        except Exception as e:
            logger.error(str(e))
            print(str(e))
            raise ToolException(str(e)) from e

    def _run(self, url: str, model_schema: str, prompt: Optional[str] = None) -> Optional[str]:
        return helpers.run_async(self._arun, url, model_schema, prompt)



import logging
import time
import os
from typing import Optional, Type

from graphlit import Graphlit
from graphlit_api import exceptions, input_types, enums
from pydantic import BaseModel, Field

from ..base_tool import BaseTool
from ..exceptions import ToolException
from .. import helpers

logger = logging.getLogger(__name__)

class GitHubIssueIngestInput(BaseModel):
    repository_name: str = Field(description="GitHub repository name")
    repository_owner: str = Field(description="GitHub repository owner")
    search: Optional[str] = Field(description="Text to search for within ingested issues", default=None)
    read_limit: Optional[int] = Field(description="Maximum number of issues from GitHub repository to be read", default=10)

class GitHubIssueIngestTool(BaseTool):
    name: str = "Graphlit GitHub Issue ingest tool"
    description: str = """Ingests issues from GitHub repository into knowledge base.
    Accepts GitHub repository owner and repository name.
    For example, for GitHub repository (https://github.com/openai/tiktoken), 'openai' is the repository owner, and 'tiktoken' is the repository name.
    Optionally accepts search text for searching within the ingested issues. If search text was not provided, all ingested issues will be returned.
    Returns extracted Markdown text and metadata from issues."""
    args_schema: Type[BaseModel] = GitHubIssueIngestInput

    graphlit: Graphlit = Field(None, exclude=True)

    workflow_id: Optional[str] = Field(None, exclude=True)
    correlation_id: Optional[str] = Field(None, exclude=True)

    model_config = {
        "arbitrary_types_allowed": True
    }

    def __init__(self, graphlit: Optional[Graphlit] = None, workflow_id: Optional[str] = None, correlation_id: Optional[str] = None, **kwargs):
        """
        Initializes the GitHubIssueIngestTool.

        Args:
            graphlit (Optional[Graphlit]): An optional Graphlit instance to interact with the Graphlit API.
                If not provided, a new Graphlit instance will be created.
            workflow_id (Optional[str]): ID for the workflow to use when ingesting issues. Defaults to None.
            correlation_id (Optional[str]): Correlation ID for tracking requests. Defaults to None.
            **kwargs: Additional keyword arguments for the BaseTool superclass.
        """
        super().__init__(**kwargs)
        self.graphlit = graphlit or Graphlit()
        self.workflow_id = workflow_id
        self.correlation_id = correlation_id

    async def _arun(self, repository_name: str, repository_owner: str, search: Optional[str] = None, read_limit: Optional[int] = None) -> Optional[str]:
        feed_id = None

        personal_access_token = os.environ['GITHUB_PERSONAL_ACCESS_TOKEN']

        if personal_access_token is None:
            raise ToolException('Invalid GitHub personal access token. Need to assign GITHUB_PERSONAL_ACCESS_TOKEN environment variable.')

        try:
            response = await self.graphlit.client.create_feed(
                feed=input_types.FeedInput(
                    name='GitHub Issue',
                    type=enums.FeedTypes.ISSUE,
                    issue=input_types.IssueFeedPropertiesInput(
                        type=enums.FeedServiceTypes.GIT_HUB_ISSUES,
                        github=input_types.GitHubIssuesFeedPropertiesInput(
                            repositoryName=repository_name,
                            repositoryOwner=repository_owner,
                            personalAccessToken=personal_access_token,
                        ),
                        readLimit=read_limit if read_limit is not None else 10
                    ),
                    workflow=input_types.EntityReferenceInput(id=self.workflow_id) if self.workflow_id is not None else None,
                ),
                correlation_id=self.correlation_id
            )

            feed_id = response.create_feed.id if response.create_feed is not None else None

            if feed_id is None:
                raise ToolException('Invalid feed identifier.')

            logger.debug(f'Created feed [{feed_id}].')

            # Wait for feed to complete, since ingestion happens asychronously
            done = False
            time.sleep(5)

            while not done:
                done = await helpers.is_feed_done(self.graphlit.client, feed_id)

                if done is None:
                    break

                if not done:
                    time.sleep(5)

            logger.debug(f'Completed feed [{feed_id}].')
        except exceptions.GraphQLClientError as e:
            logger.error(str(e))
            raise ToolException(str(e)) from e

        return await helpers.format_feed_contents(self.graphlit.client, feed_id, search)

    def _run(self, repository_name: str, repository_owner: str, search: Optional[str] = None, read_limit: Optional[int] = None) -> Optional[str]:
        return helpers.run_async(self._arun, repository_name, repository_owner, search, read_limit)



from typing import Any, cast
from .base_tool import BaseTool

CrewAIBaseTool: Any = None

try:
    from crewai.tools import BaseTool as CrewAIBaseTool
except ImportError:
    CrewAIBaseTool = None

if CrewAIBaseTool:
    class CrewAIConverter(CrewAIBaseTool):
        """Tool to convert Graphlit tools into CrewAI tools."""

        graphlit_tool: Any

        def _run(
            self,
            *args: Any,
            **kwargs: Any,
        ) -> Any:
            tool = cast(BaseTool, self.graphlit_tool)

            return tool.run(*args, **kwargs)

        async def _arun(
            self,
            *args: Any,
            **kwargs: Any,
        ) -> Any:
            tool = cast(BaseTool, self.graphlit_tool)

            return await tool.arun(*args, **kwargs)

        @classmethod
        def from_tool(cls, tool: Any, **kwargs: Any) -> "CrewAIConverter":
            if not isinstance(tool, BaseTool):
                raise ValueError(f"Expected a Graphlit tool, got {type(tool)}")

            tool = cast(BaseTool, tool)

            if tool.args_schema is None:
                raise ValueError("Invalid arguments JSON schema.")

            return cls(
                name=tool.name,
                description=tool.description,
                args_schema=tool.args_schema,
                graphlit_tool=tool,
                **kwargs,
            )
else:
    class CrewAIConverter:
        """Fallback CrewAIConverter if crewai is not installed."""
        def __init__(self, *args, **kwargs):
            raise ImportError(
                "CrewAIConverter requires the crewai package. "
                "Install it using pip install graphlit-tools[crewai]."
            )
